function PrepareTrainData()

% This function processes available illuminant and surface reflectance
% spectral distributions, resamples and projects them according to the OSA
% illumination estimation contest specifications.



%--------------------------------------------------------------------------
% Illuminant data

% Contest illuminants basis function and wavelength sampling
load('B_illum');
lambda_illum = lambdas(S_illum);
figure; plot(lambda_illum,B_illum);
title('Illuminant Basis Functions');
xlabel('Wavelength (\lambda), in nm');
ylabel('Power');

% Granada DAYLIGHT spectral database (http://www.ugr.es/~colorimg/database.html)
% J. Hernndez-Andrs, J. Romero, J.L. Nieves & R.L. Lee, Jr.
% "Color and spectral analysis of daylight in southern Europe"
% Journal of the Optical Society of America A, Vol. 18, N. 6, pp. 1325-1335, June 2001
B_granada = load('daylight_380780_eigenvectors.DAT');
illum_granada = load('daylight_standard_CCT.DAT');
S_granada = [380 5 81]; % taken from their website
illum_granada = sample_spd(illum_granada,S_granada,S_illum);

% Project onto the contest illumination basis
coeff_granada = project_spd(illum_granada,B_illum);



%--------------------------------------------------------------------------
% Surface data

% Contest surface reflectance functions (and wavelength sampling)
load('B_sur');
lambda_sur = [S_sur(1):S_sur(2):S_sur(1)+(S_sur(3)-1)*S_sur(2)]';
figure; plot(lambda_sur,B_sur);
title('Surface Reflectance Basis Functions');
xlabel('Wavelength (\lambda), in nm');
ylabel('Power');


% Nickerson, D. and D. H. Wilson (1950)
% "Munsell reference colors now specied for nine illuminants." 
% Illumination Engineering 45: 507-517
% Courtesy of Psychtoolbox distribution (http://psychtoolbox.org)
load('B_nickerson');
load('sur_nickerson');
lambda_nickerson = [S_nickerson(1):S_nickerson(2):S_nickerson(1)+(S_nickerson(3)-1)*S_nickerson(2)]';
sur_nickerson = interp1(lambda_nickerson,sur_nickerson,lambda_sur,'linear');
coeff_nickerson = project_spd(sur_nickerson,B_sur);


% Vrhel, M. J., R. Gershon, et al. (1994)
% "Measurement and analysis of object reflectance spectra." 
% Color Research And Application 19(1): 4-9
% Courtesy of Psychtoolbox distribution (http://psychtoolbox.org)
load('B_vrhel');
load('sur_vrhel');
lambda_vrhel = [S_vrhel(1):S_vrhel(2):S_vrhel(1)+(S_vrhel(3)-1)*S_vrhel(2)]';
sur_vrhel = interp1(lambda_vrhel,sur_vrhel,lambda_sur,'linear');
coeff_vrhel = project_spd(sur_vrhel,B_sur);



%--------------------------------------------------------------------------
% Finally, add the calibration data (wouldn't hurt), and write everything

load('ImageCal');
load('T_cones_osa');
load('spd_ImageCal');

coeff_cal = project_spd(spd_illumSpdCal,B_illum);
% And we might add estimated surfaces from the calibration image later...

% Put everything together
illum_coeff = [coeff_cal coeff_granada];
sur_coeff = [coeff_nickerson coeff_vrhel];

% Plot the coefficients
num_illum_coeff = size(illum_coeff,1);
num_sur_coeff = size(sur_coeff,1);
figure;
for i=1:num_illum_coeff
subplot(1,num_illum_coeff,i); hist(illum_coeff(i,:)); xlabel(sprintf('coeff%d',i));
end
title('Illuminant Spectra Coefficients');
figure;
for i=1:num_sur_coeff
subplot(1,num_sur_coeff,i); hist(sur_coeff(i,:)); xlabel(sprintf('coeff%d',i));
end
title('Surface Reflectance Coefficients');


% Write
save('train_data.mat','illum_coeff','sur_coeff');




%--------------------------------------------------------------------------

% Returns a vector of wavelengths correponding to the sampling structure
% (following the contest convension for S_* structures)
function lambda = lambdas(S_spd)
lambda = [S_spd(1):S_spd(2):S_spd(1)+(S_spd(3)-1)*S_spd(2)]';


% Resamples the spectral distribution
%   spd - the spectral distribution
%   S_spd - the sampling vector [starting wavelength , wavelength increment, num samples]
%   S_new - new sampling (same structure as S_spd)
function sampled_spd = sample_spd(spd,S_spd,S_new)
lambda_spd = lambdas(S_spd);
lambda_new = lambdas(S_new);
% The following will not really interpolate the spectra on this data (as
% all the data is given in higher sample rate than the contest sampling, and 
% and sample cycles overlap), but this is the correct way to do it in general.
sampled_spd = interp1(lambda_spd,spd,lambda_new,'linear');


% Projects spectral distribution on the given basis, assuming same
% wavelength sampling
function proj_spd = project_spd(spd,B)
proj_spd = inv(B'*B)*B'*spd;
% proj_spd = B'*spd;

